--[[
Copyright © 2014 Stefan Stryjecki

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]--

require "voronoiclass"
polygon = require "polygon"

local function initialize()
	print("generating map")
	map = VoronoiMap{maxX=wX, maxY=wY, amountOfPoints = 20000, boundingZone = 5; kmPerPixel = 20}
	print("adding heightmap")
	map:generateHeightMap(10000, 200, 0.3)
	map.seaLevel = (map.maxHeight - map.minHeight) * 0.5 + map.minHeight
	print(map.minHeight, map.seaLevel, map.maxHeight)
	map:initializeClimate(10)
	print("moving forward 1 year")
	for i=1, map.solarDays do
		map:moveForward()
		print("day", i)
		local temp = 0
		local n = 0
		for x=0, map.gridX do
			for y=0, map.gridY do
				if map.climate[x][y].temperature == map.climate[x][y].temperature then
					temp = temp + map.climate[x][y].temperature
					n = n + 1
				end
			end
		end
		print("average temperature:", temp/n)
	end
	canvas = love.graphics.newCanvas(map.maxX, map.maxY)
	love.graphics.setCanvas(canvas)
	canvas:clear()
	for i =1, #map.sites do
		if map.sites[i].height <= 0 then
			love.graphics.setColor(0, 0, 64)
		else
			local color = 255 - map.sites[i].height/(map.maxHeight-map.seaLevel) * 255
			love.graphics.setColor(color, color, color)
		end
		map:drawSite(i)
	end
	love.graphics.setCanvas()
	init = false
	init2 = false
end

local function initPrint()
	love.graphics.setColor(255,255,0)
	love.graphics.print("You can press 'space' at any time to pause the simulation and see the heightmap.", 20, wY/4)
	love.graphics.print("At the top of the screen, you will see data about the point the mouse is hovering over.", 20, wY/4 + 20)
	love.graphics.print("The red lines indicate the direction of winds.", 20, wY/4 + 40)
	love.graphics.print("Day 1 is the day of the winter solstice (December 24th)", 20, wY/4 + 60)
	love.graphics.setColor(255,255,255)
	love.graphics.print("The map is being generated. Please stand by...", 20, wY/4 + 100)
	love.graphics.print("The initial temperature map is not accurate. The system needs about a year to settle to a stable state.", 20, wY/4 + 120)
	love.graphics.print("Because of that, the first year of the simulation is done off-screen.", 20, wY/4 + 140)
	love.graphics.print("This simulation uses a lot of advanced concepts, including heat capacity, the Coriolis effect, the Earth's axial tilt, etc.", 20, wY/4 + 160)
	


	init2 = true
end

function love.load ()
	wX = 800
	wY = 600
	init = true
	goForward = true
end

function love.keypressed(key)
	if key == "m" then
		month = not month
		map:moveForward()
	elseif key == " " then
		goForward = not goForward
	elseif key == "p" then
		for k,v in pairs(map.coriolisForce) do
			print(k,v )
		end
	end
end

function love.update(dt)
	mx, my = love.mouse.getPosition()
	if init2 then 
		initialize()
	elseif map then
		if month and map.day % 30 == 0 then
			goForward = false
		elseif goForward then
			map:moveForward()
		end
	end
end


TEMPERATURECOLORS ={
	{0, 0, 102}; {0, 0, 153}; {0, 0, 204}; {0, 0, 255}; {51, 51, 255}; {102, 102, 255};
	{0, 255, 255}; {0, 255, 128}; {0, 255, 0}; {128, 255, 0}; {255, 255, 0}; {255, 128, 0};
	{255, 0, 0}; {204, 0, 0}; {128, 0, 0}; {0, 0, 0}
}

function love.draw ()
	if init then initPrint() 
	elseif map then
		love.graphics.draw(canvas)
		if goForward then
			for x=1, map.gridX do
				for y=1, map.gridY do
					local temp = math.floor((map.climate[x][y].temperature + 35) / 5) -- starting with -30C
					temp = math.min(temp, #TEMPERATURECOLORS)
					temp = math.max(temp, 1)
					local color = {unpack(TEMPERATURECOLORS[temp])}
					color[4] = 128
					love.graphics.setColor(unpack(color))
					for _, s in pairs(map.climate[x][y].sites) do
						map:drawSite(s)
					end
					love.graphics.setColor(255,0,0)
					local point = map.climate[x][y]
					love.graphics.line(point.midX, point.midY, point.midX + point.wind.x, point.midY + point.wind.y)
				end
			end
			love.graphics.setColor(255,255,255)
			local _, i = polygon.findContainingPolygon(mx, my, map.sites, "vertices")
			local site = map.sites[i]
			local text = "Day: " .. math.ceil(map.day % map.solarDays) .. ", latitude: " .. map.sites[i].latitude
			if site and site.height and site.climateGrid then
				text =  text .. ", elevation: " .. site.height .. ", temperature: "
						.. site.climateGrid.temperature
			end
			love.graphics.print(text , 0, 0)
		end
	end
end